import unittest

class DrippyTests(unittest.TestCase):

    def _getTargetClass(self):
        from drippy import Drippy
        return Drippy

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_context_no_leak(self):
        class Foo(object):
            pass
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        drippy.startContext(Foo)
        drippy.stopContext(Foo)
        self.assertEqual(len(log), 0)
        self.assertEqual(len(drippy._stack), 0)

    def test_context_no_leak_alternate_system_tempdir(self):
        import os
        class Foo(object):
            pass
        drippy = self._makeOne()
        log = []
        with _Monkey_tempfile_tempdir() as alternate:
            _before, _while = alternate
            drippy._logout = log.append
            drippy.startContext(Foo)
            not_a_leak = os.path.join(_before, 'tmp_not_a_leak')
            with open(not_a_leak, 'w') as f:
                f.write('NOT A LEAK')
            try:
                drippy.stopContext(Foo)
            finally:
                os.remove(not_a_leak)
        self.assertEqual(len(log), 0)
        self.assertEqual(len(drippy._stack), 0)

    def test_context_w_leak(self):
        from tempfile import NamedTemporaryFile
        class Foo(object):
            pass
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        drippy.startContext(Foo)
        leaky = NamedTemporaryFile()
        try:
            drippy.stopContext(Foo)
        finally:
            leaky.close()
        self.assertEqual(len(log), 1)
        self.assertEqual(len(drippy._stack), 0)

    def test_context_w_leak_alternate_system_tempdir(self):
        import os
        class Foo(object):
            pass
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        with _Monkey_tempfile_tempdir() as alternate:
            _before, _while = alternate
            drippy.startContext(Foo)
            is_a_leak = os.path.join(_while, 'tmp_is_a_leak')
            with open(is_a_leak, 'w') as f:
                f.write('IS A LEAK')
            try:
                drippy.stopContext(Foo)
            finally:
                os.remove(is_a_leak)
        self.assertEqual(len(log), 1)
        self.assertEqual(len(drippy._stack), 0)

    def test_single_test_no_leak(self):
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        drippy.beforeTest('foo')
        drippy.afterTest('foo')
        self.assertEqual(len(log), 0)
        self.assertEqual(drippy._test_count, None)

    def test_single_test_no_leak_alternate_system_tempdir(self):
        import os
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        with _Monkey_tempfile_tempdir() as alternate:
            _before, _while = alternate
            drippy.beforeTest('foo')
            not_a_leak = os.path.join(_before, 'tmp_not_a_leak')
            with open(not_a_leak, 'w') as f:
                f.write('NOT A LEAK')
            try:
                drippy.afterTest('foo')
            finally:
                os.remove(not_a_leak)
        self.assertEqual(len(log), 0)
        self.assertEqual(drippy._test_count, None)

    def test_single_test_w_leak(self):
        from tempfile import NamedTemporaryFile
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        drippy.beforeTest('foo')
        leaky = NamedTemporaryFile()
        try:
            drippy.afterTest('foo')
        finally:
            leaky.close()
        self.assertEqual(len(log), 1)
        self.assertEqual(drippy._test_count, None)

    def test_single_test_w_leak_alternate_system_tempdir(self):
        import os
        drippy = self._makeOne()
        log = []
        drippy._logout = log.append
        with _Monkey_tempfile_tempdir() as alternate:
            _before, _while = alternate
            drippy.beforeTest('foo')
            is_a_leak = os.path.join(_while, 'tmp_is_a_leak')
            with open(is_a_leak, 'w') as f:
                f.write('IS A LEAK')
            try:
                drippy.afterTest('foo')
            finally:
                os.remove(is_a_leak)
        self.assertEqual(len(log), 1)
        self.assertEqual(drippy._test_count, None)


class _Monkey_tempfile_tempdir(object):

    def __init__(self, replace='drippy'):
        import tempfile
        self._before = tempfile.tempdir
        tempfile.tempdir = self._while = tempfile.mkdtemp(prefix=replace)

    def __enter__(self):
        return self._before, self._while

    def __exit__(self, *args):
        import shutil
        import tempfile
        tempfile.tempdir = self._before
        shutil.rmtree(self._while)



import os
import sys
import tempfile

from nose.plugins import Plugin

class Drippy(Plugin):
    """ Check for tempfile leaks in tests / contexts.
    """

    _test_count = None
    _logout = None #testing hook

    def __init__(self):
        super(Drippy, self).__init__()
        self._stack = []

    def _log(self, msg, *args):
        logout =  self._logout or sys.stderr.write
        logout(msg % args)

    def startContext(self, context):
        self._stack.append((context.__name__, self._countTempfiles()))

    def stopContext(self, context):
        name, precount = self._stack.pop()
        postcount = self._countTempfiles()
        if postcount != precount:
            self._log('\n%s%s -- before: %d, after: %d\n',
                      ' ' * len(self._stack), name, precount, postcount)

    def beforeTest(self, test):
        self._test_count = self._countTempfiles()

    def afterTest(self, test):
        precount = self._test_count
        del self._test_count
        postcount = self._countTempfiles()
        if postcount != precount:
            self._log('\n%s%s -- before: %d, after: %d\n',
                      ' ' * (len(self._stack) + 1), test, precount, postcount)

    def _countTempfiles(self):
        tmp_dir = tempfile.gettempdir()
        return len([x for x in os.listdir(tmp_dir) if x.startswith('tmp')])



import os
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

setup(
    name='drippy',
    version='0.3.1',
    description='Nose plugin for finding tempfile leaks',
    long_description='\n\n'.join([README, CHANGES]),
    platforms=['Unix', 'Windows'],
    keywords='nose tempfile leak',
    url='https://launchpad.net/drippy',
    author='Tres Seaver, Agendaless Consulting',
    author_email='tseaver@agendaless.com',
    license='Python',
    classifiers=[
      'Intended Audience :: Developers',
      'License :: OSI Approved :: Python Software Foundation License',
      'Operating System :: OS Independent',
      'Programming Language :: Python',
      'Programming Language :: Python :: 2',
      'Programming Language :: Python :: 2.6',
      'Programming Language :: Python :: 2.7',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.2',
      'Programming Language :: Python :: 3.3',
      'Programming Language :: Python :: Implementation :: CPython',
      'Programming Language :: Python :: Implementation :: PyPy',
      'Topic :: Software Development :: Testing',
    ],
    install_requires=['nose'],
    packages=['drippy'],
    test_suite='drippy.tests',
    entry_points = {
        'nose.plugins': [
            'hack = drippy:Drippy',
        ]
    },
)
